// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#ifndef ECM_QLOGGINGCATEGORY_PLASMAPASS_PLASMAPASS_LOG_PLASMAPASS_DEBUG_H
#define ECM_QLOGGINGCATEGORY_PLASMAPASS_PLASMAPASS_LOG_PLASMAPASS_DEBUG_H

#include <QLoggingCategory>
 namespace PlasmaPass {
Q_DECLARE_LOGGING_CATEGORY(PLASMAPASS_LOG)
} 

#endif
