/****************************************************************************
** Meta object code from reading C++ file 'klipperinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "klipperinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'klipperinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKlipperKlipperInterface::qt_create_metaobjectdata<qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKlipperKlipperInterface",
        "clearClipboardContents",
        "QDBusPendingReply<>",
        "",
        "clearClipboardHistory",
        "getClipboardContents",
        "QDBusPendingReply<QString>",
        "getClipboardHistoryItem",
        "i",
        "getClipboardHistoryMenu",
        "QDBusPendingReply<QStringList>",
        "saveClipboardHistory",
        "setClipboardContents",
        "s",
        "showKlipperManuallyInvokeActionMenu",
        "showKlipperPopupMenu"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'clearClipboardContents'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'clearClipboardHistory'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(4, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'getClipboardContents'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(5, 3, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'getClipboardHistoryItem'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(int)>(7, 3, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'getClipboardHistoryMenu'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(9, 3, QMC::AccessPublic, 0x80000000 | 10),
        // Slot 'saveClipboardHistory'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(11, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'setClipboardContents'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(12, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'showKlipperManuallyInvokeActionMenu'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(14, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'showKlipperPopupMenu'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(15, 3, QMC::AccessPublic, 0x80000000 | 2),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKlipperKlipperInterface, qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKlipperKlipperInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKlipperKlipperInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKlipperKlipperInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->clearClipboardContents();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->clearClipboardHistory();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QString> _r = _t->getClipboardContents();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->getClipboardHistoryItem((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QStringList> _r = _t->getClipboardHistoryMenu();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->saveClipboardHistory();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->setClipboardContents((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->showKlipperManuallyInvokeActionMenu();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->showKlipperPopupMenu();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *OrgKdeKlipperKlipperInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKlipperKlipperInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29OrgKdeKlipperKlipperInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKlipperKlipperInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
