/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.klipper.klipper.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef KLIPPERINTERFACE_H
#define KLIPPERINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.klipper.klipper
 */
class OrgKdeKlipperKlipperInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.klipper.klipper"; }

public:
    OrgKdeKlipperKlipperInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKlipperKlipperInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> clearClipboardContents()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("clearClipboardContents"), argumentList);
    }

    inline QDBusPendingReply<> clearClipboardHistory()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("clearClipboardHistory"), argumentList);
    }

    inline QDBusPendingReply<QString> getClipboardContents()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("getClipboardContents"), argumentList);
    }

    inline QDBusPendingReply<QString> getClipboardHistoryItem(int i)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(i);
        return asyncCallWithArgumentList(QStringLiteral("getClipboardHistoryItem"), argumentList);
    }

    inline QDBusPendingReply<QStringList> getClipboardHistoryMenu()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("getClipboardHistoryMenu"), argumentList);
    }

    inline QDBusPendingReply<> saveClipboardHistory()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("saveClipboardHistory"), argumentList);
    }

    inline QDBusPendingReply<> setClipboardContents(const QString &s)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(s);
        return asyncCallWithArgumentList(QStringLiteral("setClipboardContents"), argumentList);
    }

    inline QDBusPendingReply<> showKlipperManuallyInvokeActionMenu()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("showKlipperManuallyInvokeActionMenu"), argumentList);
    }

    inline QDBusPendingReply<> showKlipperPopupMenu()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("showKlipperPopupMenu"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace kde {
    namespace klipper {
      using klipper = ::OrgKdeKlipperKlipperInterface;
    }
  }
}

#endif
