// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plasmapass_debug.h"

 namespace PlasmaPass {
Q_LOGGING_CATEGORY(PLASMAPASS_LOG, "org.kde.plasma.pass", QtInfoMsg)
} 
